#  tests for sputnik-0.9.3-py39hde42818_1002 (this is a generated file);
print('===== testing package: sputnik-0.9.3-py39hde42818_1002 =====');
print('running run_test.py');
#  --- run_test.py (begin) ---
import os
import pytest
import sputnik

PACKAGE_DIR = os.path.abspath(os.path.dirname(sputnik.__file__))
pytest.main([PACKAGE_DIR])
#  --- run_test.py (end) ---

print('===== sputnik-0.9.3-py39hde42818_1002 OK =====');
